"""Shared fixtures for aiohasupervisor tests."""

from collections.abc import AsyncGenerator, Generator

from aioresponses import aioresponses
import pytest

from aiohasupervisor import SupervisorClient

from .const import SUPERVISOR_URL


@pytest.fixture(name="supervisor_client")
async def client() -> AsyncGenerator[SupervisorClient, None]:
    """Return a Supervisor client."""
    async with SupervisorClient(
        SUPERVISOR_URL,
        "abc123",
    ) as supervisor_client:
        yield supervisor_client


@pytest.fixture(name="responses")
def aioresponses_fixture() -> Generator[aioresponses, None, None]:
    """Return aioresponses fixture."""
    with aioresponses() as mocked_responses:
        yield mocked_responses



"""Test mounts supervisor client."""

from pathlib import PurePath

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import (
    CIFSMountRequest,
    MountCifsVersion,
    MountsOptions,
    MountUsage,
    NFSMountRequest,
)

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_mounts_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test mounts info API."""
    responses.get(
        f"{SUPERVISOR_URL}/mounts", status=200, body=load_fixture("mounts_info.json")
    )
    info = await supervisor_client.mounts.info()
    assert info.default_backup_mount == "Test"

    assert info.mounts[0].name == "Test"
    assert info.mounts[0].server == "test.local"
    assert info.mounts[0].type == "cifs"
    assert info.mounts[0].share == "backup"
    assert info.mounts[0].usage == "backup"
    assert info.mounts[0].read_only is False
    assert info.mounts[0].version is None
    assert info.mounts[0].state == "active"
    assert info.mounts[0].user_path is None

    assert info.mounts[1].usage == "share"
    assert info.mounts[1].read_only is True
    assert info.mounts[1].version == "2.0"
    assert info.mounts[1].port == 12345
    assert info.mounts[1].user_path == PurePath("/share/Test2")

    assert info.mounts[2].type == "nfs"
    assert info.mounts[2].usage == "media"
    assert info.mounts[2].path.as_posix() == "media"
    assert info.mounts[2].user_path == PurePath("/media/Test3")


@pytest.mark.parametrize("mount_name", ["test", None])
async def test_mounts_options(
    responses: aioresponses, supervisor_client: SupervisorClient, mount_name: str | None
) -> None:
    """Test mounts options API."""
    responses.post(f"{SUPERVISOR_URL}/mounts/options", status=200)
    assert (
        await supervisor_client.mounts.options(
            MountsOptions(default_backup_mount=mount_name)
        )
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/mounts/options"))
    }


@pytest.mark.parametrize(
    "mount_config",
    [
        CIFSMountRequest(
            server="test.local",
            share="backup",
            usage=MountUsage.BACKUP,
        ),
        CIFSMountRequest(
            server="test.local",
            share="media",
            port=12345,
            usage=MountUsage.MEDIA,
            version=MountCifsVersion.LEGACY_2_0,
            read_only=True,
            username="test",
            password="test",  # noqa: S106
        ),
        NFSMountRequest(
            server="test.local",
            path=PurePath("share"),
            usage=MountUsage.SHARE,
        ),
        NFSMountRequest(
            server="test.local",
            path=PurePath("backups"),
            port=12345,
            read_only=False,
            usage=MountUsage.BACKUP,
        ),
    ],
)
async def test_create_mount(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    mount_config: CIFSMountRequest | NFSMountRequest,
) -> None:
    """Test create mount API."""
    responses.post(f"{SUPERVISOR_URL}/mounts", status=200)
    assert await supervisor_client.mounts.create_mount("test", mount_config) is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/mounts"))}


@pytest.mark.parametrize(
    "mount_config",
    [
        CIFSMountRequest(
            server="test.local",
            share="backup",
            usage=MountUsage.BACKUP,
        ),
        CIFSMountRequest(
            server="test.local",
            share="media",
            port=12345,
            usage=MountUsage.MEDIA,
            version=MountCifsVersion.LEGACY_2_0,
            read_only=True,
            username="test",
            password="test",  # noqa: S106
        ),
        NFSMountRequest(
            server="test.local",
            path=PurePath("share"),
            usage=MountUsage.SHARE,
        ),
        NFSMountRequest(
            server="test.local",
            path=PurePath("backups"),
            port=12345,
            read_only=False,
            usage=MountUsage.BACKUP,
        ),
    ],
)
async def test_update_mount(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    mount_config: CIFSMountRequest | NFSMountRequest,
) -> None:
    """Test update mount API."""
    responses.put(f"{SUPERVISOR_URL}/mounts/test", status=200)
    assert await supervisor_client.mounts.update_mount("test", mount_config) is None
    assert responses.requests.keys() == {("PUT", URL(f"{SUPERVISOR_URL}/mounts/test"))}


async def test_delete_mount(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test delete mount API."""
    responses.delete(f"{SUPERVISOR_URL}/mounts/test", status=200)
    assert await supervisor_client.mounts.delete_mount("test") is None
    assert responses.requests.keys() == {
        ("DELETE", URL(f"{SUPERVISOR_URL}/mounts/test"))
    }


async def test_reload_mount(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test reload mount API."""
    responses.post(f"{SUPERVISOR_URL}/mounts/test/reload", status=200)
    assert await supervisor_client.mounts.reload_mount("test") is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/mounts/test/reload"))
    }



"""Models for discovery component."""

from dataclasses import dataclass
from typing import Any
from uuid import UUID

from .base import Request, ResponseData


@dataclass(frozen=True, slots=True)
class DiscoveryConfig(Request):
    """DiscoveryConfig model."""

    service: str
    config: dict[str, Any]


@dataclass(frozen=True, slots=True)
class Discovery(ResponseData):
    """Discovery model."""

    addon: str
    service: str
    uuid: UUID
    config: dict[str, Any]


@dataclass(frozen=True, slots=True)
class DiscoveryList(ResponseData):
    """DiscoveryList model."""

    discovery: list[Discovery]


@dataclass(frozen=True, slots=True)
class SetDiscovery(ResponseData):
    """SetDiscovery model."""

    uuid: UUID



"""Tests for store supervisor client."""

from aioresponses import aioresponses
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import StoreAddonUpdate, StoreAddRepository

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_store_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store info API."""
    responses.get(
        f"{SUPERVISOR_URL}/store", status=200, body=load_fixture("store_info.json")
    )
    store = await supervisor_client.store.info()

    assert len(store.repositories) == 6
    assert store.repositories[0].slug == "local"
    assert store.repositories[0].name == "Local add-ons"

    assert len(store.addons) == 88
    assert store.addons[0].slug == "d5369777_music_assistant"
    assert store.addons[0].name == "Music Assistant Server"


async def test_store_addons_list(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addons list API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons",
        status=200,
        body=load_fixture("store_addons_list.json"),
    )
    addons = await supervisor_client.store.addons_list()

    assert len(addons) == 88
    assert addons[0].slug == "d5369777_music_assistant"
    assert addons[0].name == "Music Assistant Server"


async def test_store_repositories_list(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store repositories list API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/repositories",
        status=200,
        body=load_fixture("store_repositories_list.json"),
    )
    repositories = await supervisor_client.store.repositories_list()

    assert len(repositories) == 6
    assert repositories[0].slug == "local"
    assert repositories[0].name == "Local add-ons"


async def test_store_addon_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon info API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto",
        status=200,
        body=load_fixture("store_addon_info.json"),
    )
    info = await supervisor_client.store.addon_info("core_mosquitto")

    assert info.name == "Mosquitto broker"
    assert info.slug == "core_mosquitto"
    assert info.signed is True
    assert info.supervisor_api is False
    assert info.supervisor_role == "default"


async def test_store_addon_changelog(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon info changelog API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/changelog",
        status=200,
        body=load_fixture("store_addon_changelog.txt"),
    )
    changelog = await supervisor_client.store.addon_changelog("core_mosquitto")
    assert changelog.startswith("# Changelog")


async def test_store_addon_documentation(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon documentation API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/documentation",
        status=200,
        body=load_fixture("store_addon_documentation.txt"),
    )
    documentation = await supervisor_client.store.addon_documentation("core_mosquitto")
    assert documentation.startswith("# Home Assistant Add-on: Mosquitto broker")


async def test_store_addon_install(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon install API."""
    responses.post(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/install", status=200)

    assert (await supervisor_client.store.install_addon("core_mosquitto")) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/install"))
    }


async def test_store_addon_update(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon update API."""
    responses.post(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/update", status=200)

    assert (
        await supervisor_client.store.update_addon(
            "core_mosquitto", StoreAddonUpdate(backup=True)
        )
    ) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/update"))
    }


async def test_store_reload(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store reload API."""
    responses.post(f"{SUPERVISOR_URL}/store/reload", status=200)

    assert (await supervisor_client.store.reload()) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/store/reload"))
    }


async def test_store_repository_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store repository info API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/repositories/core",
        status=200,
        body=load_fixture("store_repository_info.json"),
    )
    repository = await supervisor_client.store.repository_info("core")
    assert repository.slug == "core"
    assert repository.name == "Official add-ons"
    assert repository.source == "core"


async def test_store_add_repository(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store add repository API."""
    responses.post(f"{SUPERVISOR_URL}/store/repositories", status=200)

    assert (
        await supervisor_client.store.add_repository(
            StoreAddRepository(repository="test")
        )
    ) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/store/repositories"))
    }


async def test_store_remove_repository(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon info API."""
    responses.delete(f"{SUPERVISOR_URL}/store/repositories/test", status=200)

    assert (await supervisor_client.store.remove_repository("test")) is None
    assert responses.requests.keys() == {
        ("DELETE", URL(f"{SUPERVISOR_URL}/store/repositories/test"))
    }



"""Models for Supervisor addons."""

from abc import ABC
from dataclasses import dataclass, field
from enum import StrEnum
from ipaddress import IPv4Address
from typing import Any

from mashumaro import field_options
from mashumaro.config import TO_DICT_ADD_BY_ALIAS_FLAG, BaseConfig

from .base import DEFAULT, ContainerStats, Options, Request, RequestConfig, ResponseData

# --- ENUMS ----


class AddonStage(StrEnum):
    """AddonStage type."""

    STABLE = "stable"
    EXPERIMENTAL = "experimental"
    DEPRECATED = "deprecated"


class AddonBootConfig(StrEnum):
    """AddonBootConfig type."""

    AUTO = "auto"
    MANUAL = "manual"
    MANUAL_ONLY = "manual_only"


class AddonBoot(StrEnum):
    """AddonBoot type."""

    AUTO = "auto"
    MANUAL = "manual"


class CpuArch(StrEnum):
    """CpuArch type."""

    ARMV7 = "armv7"
    ARMHF = "armhf"
    AARCH64 = "aarch64"
    I386 = "i386"
    AMD64 = "amd64"


class Capability(StrEnum):
    """Capability type.

    This is an incomplete list. Supervisor regularly adds support for new
    privileged capabilities as addon developers request them. Therefore
    when returning a list of capabilities, there may be some which are not in
    this list parsed as strings on older versions of the client.
    """

    BPF = "BPF"
    DAC_READ_SEARCH = "DAC_READ_SEARCH"
    IPC_LOCK = "IPC_LOCK"
    NET_ADMIN = "NET_ADMIN"
    NET_RAW = "NET_RAW"
    PERFMON = "PERFMON"
    SYS_ADMIN = "SYS_ADMIN"
    SYS_MODULE = "SYS_MODULE"
    SYS_NICE = "SYS_NICE"
    SYS_PTRACE = "SYS_PTRACE"
    SYS_RAWIO = "SYS_RAWIO"
    SYS_RESOURCE = "SYS_RESOURCE"
    SYS_TIME = "SYS_TIME"


class AppArmor(StrEnum):
    """AppArmor type."""

    DEFAULT = "default"
    DISABLE = "disable"
    PROFILE = "profile"


class SupervisorRole(StrEnum):
    """SupervisorRole type."""

    ADMIN = "admin"
    BACKUP = "backup"
    DEFAULT = "default"
    HOMEASSISTANT = "homeassistant"
    MANAGER = "manager"


class AddonState(StrEnum):
    """AddonState type."""

    STARTUP = "startup"
    STARTED = "started"
    STOPPED = "stopped"
    UNKNOWN = "unknown"
    ERROR = "error"


class AddonStartup(StrEnum):
    """AddonStartup type."""

    APPLICATION = "application"
    INITIALIZE = "initialize"
    ONCE = "once"
    SERVICES = "services"
    SYSTEM = "system"


# --- OBJECTS ----


@dataclass(frozen=True)
class AddonInfoBaseFields(ABC):
    """AddonInfoBaseFields ABC type."""

    advanced: bool
    available: bool
    build: bool
    description: str
    homeassistant: str | None
    icon: bool
    logo: bool
    name: str
    repository: str
    slug: str
    stage: AddonStage
    update_available: bool
    url: str | None
    version_latest: str
    version: str | None


@dataclass(frozen=True)
class AddonInfoStoreBaseFields(ABC):
    """AddonInfoAllStoreFields ABC type."""

    arch: list[CpuArch]
    documentation: bool


@dataclass(frozen=True)
class AddonInfoStoreExtFields(ABC):
    """AddonInfoStoreExtFields ABC type."""

    apparmor: AppArmor
    auth_api: bool
    docker_api: bool
    full_access: bool
    homeassistant_api: bool
    host_network: bool
    host_pid: bool
    ingress: bool
    long_description: str | None
    rating: int
    signed: bool

    # Hassio is deprecated name for supervisor
    supervisor_api: bool = field(metadata=field_options(alias="hassio_api"))
    supervisor_role: SupervisorRole = field(
        metadata=field_options(alias="hassio_role"),
    )

    class Config(BaseConfig):
        """Mashumaro config options."""

        code_generation_options = [TO_DICT_ADD_BY_ALIAS_FLAG]  # noqa: RUF012


@dataclass(frozen=True)
class AddonInfoStoreExtInstalledBaseFields(ABC):
    """AddonInfoStoreExtInstalledBaseFields ABC type."""

    detached: bool


@dataclass(frozen=True, slots=True)
class StoreAddon(AddonInfoBaseFields, AddonInfoStoreBaseFields, ResponseData):
    """StoreAddon type."""

    installed: bool


@dataclass(frozen=True, slots=True)
class StoreAddonComplete(
    AddonInfoBaseFields,
    AddonInfoStoreBaseFields,
    AddonInfoStoreExtFields,
    AddonInfoStoreExtInstalledBaseFields,
    ResponseData,
):
    """StoreAddonComplete type."""

    installed: bool


@dataclass(frozen=True, slots=True)
class InstalledAddon(
    AddonInfoBaseFields,
    AddonInfoStoreExtInstalledBaseFields,
    ResponseData,
):
    """InstalledAddon type."""

    state: AddonState


@dataclass(frozen=True, slots=True)
class InstalledAddonComplete(
    AddonInfoBaseFields,
    AddonInfoStoreExtInstalledBaseFields,
    AddonInfoStoreBaseFields,
    AddonInfoStoreExtFields,
    ResponseData,
):
    """InstalledAddonComplete model."""

    state: AddonState
    hostname: str
    dns: list[str]
    protected: bool
    boot: AddonBoot
    boot_config: AddonBootConfig
    options: dict[str, Any]
    schema: list[dict[str, Any]] | None
    machine: list[str]
    network: dict[str, int | None] | None
    network_description: dict[str, str] | None
    host_ipc: bool
    host_uts: bool
    host_dbus: bool
    privileged: list[Capability | str]
    changelog: bool
    stdin: bool
    gpio: bool
    usb: bool
    uart: bool
    kernel_modules: bool
    devicetree: bool
    udev: bool
    video: bool
    audio: bool
    startup: AddonStartup
    services: list[str]
    discovery: list[str]
    translations: dict[str, Any]
    webui: str | None
    ingress_entry: str | None
    ingress_url: str | None
    ingress_port: int | None
    ingress_panel: bool | None
    audio_input: str | None
    audio_output: str | None
    auto_update: bool
    ip_address: IPv4Address
    watchdog: bool
    devices: list[str]
    system_managed: bool
    system_managed_config_entry: str | None


@dataclass(frozen=True, slots=True)
class AddonsList(ResponseData):
    """AddonsList model."""

    addons: list[InstalledAddon]


@dataclass(frozen=True, slots=True)
class AddonsOptions(Options):
    """AddonsOptions model."""

    # Options term is used to reference both general options and addon-specific config
    # Therefore this field is config to match UI rather then Supervisor's API
    config: dict[str, Any] | None = field(  # type: ignore[assignment]
        default=DEFAULT,
        metadata=field_options(alias="options"),
    )
    boot: AddonBoot | None = None
    auto_update: bool | None = None
    network: dict[str, int | None] | None = DEFAULT  # type: ignore[assignment]
    audio_input: str | None = DEFAULT  # type: ignore[assignment]
    audio_output: str | None = DEFAULT  # type: ignore[assignment]
    ingress_panel: bool | None = None
    watchdog: bool | None = None

    class Config(RequestConfig):
        """Mashumaro config options."""

        serialize_by_alias = True


@dataclass(frozen=True, slots=True)
class AddonsConfigValidate(ResponseData):
    """AddonsConfigValidate model."""

    message: str
    valid: bool
    pwned: bool | None


@dataclass(frozen=True, slots=True)
class AddonsSecurityOptions(Options):
    """AddonsSecurityOptions model."""

    protected: bool | None = None


@dataclass(frozen=True, slots=True)
class AddonsStats(ContainerStats):
    """AddonsStats model."""


@dataclass(frozen=True, slots=True)
class AddonsUninstall(Request):
    """AddonsUninstall model."""

    remove_config: bool | None = None


@dataclass(frozen=True, slots=True)
class Repository(ResponseData):
    """Repository model."""

    slug: str
    name: str
    source: str
    url: str
    maintainer: str


@dataclass(frozen=True, slots=True)
class StoreAddonsList(ResponseData):
    """StoreAddonsList model."""

    addons: list[StoreAddon]


@dataclass(frozen=True, slots=True)
class StoreInfo(StoreAddonsList, ResponseData):
    """StoreInfo model."""

    repositories: list[Repository]


@dataclass(frozen=True, slots=True)
class StoreAddonUpdate(Request):
    """StoreAddonUpdate model."""

    backup: bool | None = None


@dataclass(frozen=True, slots=True)
class StoreAddRepository(Request):
    """StoreAddRepository model."""

    repository: str



"""Models for Supervisor jobs."""

from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime  # noqa: TCH003
from enum import StrEnum
from uuid import UUID  # noqa: TCH003

from .base import Request, ResponseData

# --- ENUMS ----


class JobCondition(StrEnum):
    """JobCondition type.

    This is an incomplete list. Supervisor regularly adds support for new
    job conditions as they are found to be needed. Therefore when returning
    a list of job conditions, there may be some which are not in
    this list parsed as strings on older versions of the client.
    """

    AUTO_UPDATE = "auto_update"
    FREE_SPACE = "free_space"
    FROZEN = "frozen"
    HAOS = "haos"
    HEALTHY = "healthy"
    HOST_NETWORK = "host_network"
    INTERNET_HOST = "internet_host"
    INTERNET_SYSTEM = "internet_system"
    MOUNT_AVAILABLE = "mount_available"
    OS_AGENT = "os_agent"
    PLUGINS_UPDATED = "plugins_updated"
    RUNNING = "running"
    SUPERVISOR_UPDATED = "supervisor_updated"


# --- OBJECTS ----


@dataclass(slots=True, frozen=True)
class JobError(ResponseData):
    """JobError model."""

    type: str
    message: str


@dataclass(slots=True, frozen=True)
class Job(ResponseData):
    """Job model."""

    name: str | None
    reference: str | None
    uuid: UUID
    progress: float
    stage: str | None
    done: bool | None
    errors: list[JobError]
    created: datetime
    child_jobs: list[Job]


@dataclass(slots=True, frozen=True)
class JobsInfo(ResponseData):
    """JobsInfo model."""

    ignore_conditions: list[JobCondition | str]
    jobs: list[Job]


@dataclass(slots=True, frozen=True)
class JobsOptions(Request):
    """JobsOptions model."""

    # We only do `| str` in responses since we can't control what supervisor returns
    # Support for ignoring new job conditions will wait for a new version of library
    ignore_conditions: list[JobCondition]



"""Test OS supervisor client."""

from pathlib import PurePath

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import (
    BootSlotName,
    GreenOptions,
    MigrateDataOptions,
    OSUpdate,
    SetBootSlotOptions,
    YellowOptions,
)
from aiohasupervisor.models.os import SwapOptions

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_os_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS info API."""
    responses.get(
        f"{SUPERVISOR_URL}/os/info",
        status=200,
        body=load_fixture("os_info.json"),
    )
    info = await supervisor_client.os.info()
    assert info.version == "13.0"
    assert info.version_latest == "13.1"
    assert info.update_available is True
    assert info.boot_slots["A"].state == "inactive"
    assert info.boot_slots["B"].state == "booted"
    assert info.boot_slots["B"].status == "good"
    assert info.boot_slots["B"].version == "13.0"


@pytest.mark.parametrize("options", [None, OSUpdate(version="13.0")])
async def test_os_update(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: OSUpdate | None,
) -> None:
    """Test OS update API."""
    responses.post(f"{SUPERVISOR_URL}/os/update", status=200)
    assert await supervisor_client.os.update(options) is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/os/update"))}


async def test_os_swap_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS config swap API."""
    responses.get(
        f"{SUPERVISOR_URL}/os/config/swap",
        status=200,
        body=load_fixture("os_config_swap.json"),
    )
    info = await supervisor_client.os.swap_info()
    assert info.swap_size == "1G"
    assert info.swappiness == 1


async def test_os_set_swap_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS set swap options API."""
    responses.post(f"{SUPERVISOR_URL}/os/config/swap", status=200)
    assert (
        await supervisor_client.os.set_swap_options(
            SwapOptions(swap_size="1G", swappiness=20)
        )
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/os/config/swap"))
    }


async def test_os_config_sync(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS config sync API."""
    responses.post(f"{SUPERVISOR_URL}/os/config/sync", status=200)
    assert await supervisor_client.os.config_sync() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/os/config/sync"))
    }


async def test_os_migrate_data(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS migrate data API."""
    responses.post(f"{SUPERVISOR_URL}/os/datadisk/move", status=200)
    assert (
        await supervisor_client.os.migrate_data(MigrateDataOptions(device="/dev/test"))
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/os/datadisk/move"))
    }


async def test_os_list_data_disks(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS datadisk list API."""
    responses.get(
        f"{SUPERVISOR_URL}/os/datadisk/list",
        status=200,
        body=load_fixture("os_datadisk_list.json"),
    )
    datadisks = await supervisor_client.os.list_data_disks()
    assert datadisks[0].vendor == "SSK"
    assert datadisks[0].model == "SSK Storage"
    assert datadisks[0].serial == "DF123"
    assert datadisks[0].name == "SSK SSK Storage (DF123)"
    assert datadisks[0].dev_path == PurePath("/dev/sda")


async def test_os_wipe_data(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS wipe data API."""
    responses.post(f"{SUPERVISOR_URL}/os/datadisk/wipe", status=200)
    assert await supervisor_client.os.wipe_data() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/os/datadisk/wipe"))
    }


async def test_os_set_boot_slot(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS set boot slot API."""
    responses.post(f"{SUPERVISOR_URL}/os/boot-slot", status=200)
    assert (
        await supervisor_client.os.set_boot_slot(
            SetBootSlotOptions(boot_slot=BootSlotName.B)
        )
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/os/boot-slot"))
    }


async def test_os_green_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS green board info API."""
    responses.get(
        f"{SUPERVISOR_URL}/os/boards/green",
        status=200,
        body=load_fixture("os_green_info.json"),
    )
    info = await supervisor_client.os.green_info()
    assert info.activity_led is True
    assert info.power_led is True
    assert info.system_health_led is True


async def test_os_green_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS green board options API."""
    responses.post(f"{SUPERVISOR_URL}/os/boards/green", status=200)
    assert (
        await supervisor_client.os.set_green_options(GreenOptions(activity_led=False))
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/os/boards/green"))
    }


async def test_os_yellow_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS yellow board info API."""
    responses.get(
        f"{SUPERVISOR_URL}/os/boards/yellow",
        status=200,
        body=load_fixture("os_yellow_info.json"),
    )
    info = await supervisor_client.os.yellow_info()
    assert info.disk_led is True
    assert info.heartbeat_led is True
    assert info.power_led is True


async def test_os_yellow_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test OS yellow board options API."""
    responses.post(f"{SUPERVISOR_URL}/os/boards/yellow", status=200)
    assert (
        await supervisor_client.os.set_yellow_options(
            YellowOptions(heartbeat_led=False)
        )
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/os/boards/yellow"))
    }



"""Models for supervisor component."""

from dataclasses import dataclass
from ipaddress import IPv4Address

from .base import ContainerStats, Options, Request, ResponseData
from .root import LogLevel, UpdateChannel


@dataclass(frozen=True, slots=True)
class SupervisorInfo(ResponseData):
    """SupervisorInfo model."""

    version: str
    version_latest: str
    update_available: bool
    channel: UpdateChannel
    arch: str
    supported: bool
    healthy: bool
    ip_address: IPv4Address
    timezone: str | None
    logging: LogLevel
    debug: bool
    debug_block: bool
    diagnostics: bool | None
    auto_update: bool
    country: str | None


@dataclass(frozen=True, slots=True)
class SupervisorStats(ContainerStats):
    """SupervisorStats model."""


@dataclass(frozen=True, slots=True)
class SupervisorUpdateOptions(Request):
    """SupervisorUpdateOptions model."""

    version: str


@dataclass(frozen=True, slots=True)
class SupervisorOptions(Options):
    """SupervisorOptions model."""

    channel: UpdateChannel | None = None
    timezone: str | None = None
    logging: LogLevel | None = None
    debug: bool | None = None
    debug_block: bool | None = None
    diagnostics: bool | None = None
    content_trust: bool | None = None
    force_security: bool | None = None
    auto_update: bool | None = None
